package org.eclipse.wtp.releng.cvslog.tools;

import java.util.HashMap;
import java.util.Map;

import com.meterware.httpunit.GetMethodWebRequest;
import com.meterware.httpunit.TableCell;
import com.meterware.httpunit.WebConversation;
import com.meterware.httpunit.WebForm;
import com.meterware.httpunit.WebRequest;
import com.meterware.httpunit.WebResponse;
import com.meterware.httpunit.WebTable;

public class BugzillaUtil {

	/**
	 * 
	 * URL of bugzillabug
	 */
	private static String BUGZILLA_URL = "https://bugs.eclipse.org/bugs/show_bug.cgi?id=";

	/**
	 * id of the form in the bug page
	 */
	private static String BUGZILLA_FORM = "changeform";

	private static Map bugzillaCache = new HashMap();

	/**
	 * id list of the field in the bug page
	 */
	public static final String BUGZILLA_SUMMARY = "short_desc";

	public static final String BUGZILLA_SEVERITY = "bug_severity";

	public static final String BUGZILLA_PRIORITY = "priority";

	public static final String BUGZILLA_HARDWARE = "rep_platform";

	public static final String BUGZILLA_VERSION = "version";

	public static final String BUGZILLA_TARGET_MILESTONE = "target_milestone";

	public static final String BUGZILLA_OS = "op_sys";

	public static final String BUGZILLA_STATUS = "status";

	public static final String BUGZILLA_RESOLUTION = "resolution";

	public static final String BUGZILLA_ASSIGNTO = "assignedTo";


	/**
	 * Connects to the bugzilla and gets required datas from the bug's page
	 * 
	 * @param input
	 *            Bug No String
	 * @return Map of bug properties that retrieved forn bugzilla.
	 */

	public static Map getBugzillaData(String bugId) {

		Map bugzillaData = new HashMap();
		WebResponse response;

		try {

			if (bugzillaCache.containsKey(bugId)) {
				// System.out.println("get data from cache for bug : "+bugId);
				return (Map) bugzillaCache.get(bugId);
			}
			Trace.consoleLogBug(bugId);
			WebConversation wc = new WebConversation();
			WebRequest request = new GetMethodWebRequest(BUGZILLA_URL + bugId);
			response = wc.getResponse(request);

			WebForm form = response.getFormWithName(BUGZILLA_FORM);
			WebTable table = response.getTables()[1];
			TableCell tableCell = table.getTableCell(0, 0);
			WebTable[] tables = tableCell.getTables();
			if (tables != null && tables.length > 0) {
				WebTable tableCellTable = tables[0];// table containing status
													// and resolution of bug

				bugzillaData.put(BUGZILLA_HARDWARE, form
						.getParameterValue(BUGZILLA_HARDWARE));
				bugzillaData.put(BUGZILLA_OS, form
						.getParameterValue(BUGZILLA_OS));
				bugzillaData.put(BUGZILLA_PRIORITY, form
						.getParameterValue(BUGZILLA_PRIORITY));
				bugzillaData.put(BUGZILLA_SEVERITY, form
						.getParameterValue(BUGZILLA_SEVERITY));
				bugzillaData.put(BUGZILLA_SUMMARY, form
						.getParameterValue(BUGZILLA_SUMMARY));
				bugzillaData.put(BUGZILLA_TARGET_MILESTONE, form
						.getParameterValue(BUGZILLA_TARGET_MILESTONE));
				bugzillaData.put(BUGZILLA_VERSION, form
						.getParameterValue(BUGZILLA_VERSION));
				bugzillaData.put(BUGZILLA_STATUS, tableCellTable.getCellAsText(
						3, 1));
				bugzillaData.put(BUGZILLA_RESOLUTION, tableCellTable
						.getCellAsText(4, 1));

				bugzillaData.put(BUGZILLA_ASSIGNTO, tableCellTable
						.getCellAsText(5, 1));
				bugzillaCache.put(bugId, bugzillaData);
			}

		} catch (Exception e) {
			Trace.consoleLogBug(" BugzillaException["+bugId+"]");
			//e.printStackTrace();
		}

		return bugzillaData;
	}

}
